from analyses.analysisbase import tests as base_tests

from analyses.dispersions import models as app_models

from django.contrib.auth.models import User

class PermissionsMixIn(object):
    def setPermissions(self):
        from django.contrib.auth.models import User, Permission
        from django.contrib.contenttypes.models import ContentType
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = unauthorised to edit
        # 3 = unauthorised to even display

        u = User.objects.get(pk=1)

        permissions = [
            ('display_versions','versioning','version'),
            ('display_published_versions','versioning','version'),
            ('edit_versions','versioning','version'),
            ('display_dispersionscenarios','dispersions','dispersionscenario'),
            ('edit_dispersionscenarios','dispersions','dispersionscenario'),
            ('display_installations','installations','section'),
            ('display_dispersionsuggestionlists','dispersions','dispersionsuggestionlist'),
            ('edit_dispersionsuggestionlists','dispersions','dispersionsuggestionlist'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=2)
        permissions = [
            ('display_versions','versioning','version'),
            ('display_published_versions','versioning','version'),
            ('display_dispersionscenarios','dispersions','dispersionscenario'),
            ('display_installations','installations','section'),
            ('display_dispersionsuggestionlists','dispersions','dispersionsuggestionlist'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

class SettingsMixIn(object):
    sc_model = app_models.DispersionScenario
    ev_model = app_models.DispersionEvent
    pl_model = app_models.DispersionProtectionLayer
    me_model = app_models.DispersionMeasure
    plme_model = app_models.DispersionProtectionLayer_Measure
    ap_model = app_models.DispersionAttentionPoint
    sl_model = app_models.DispersionSuggestionlist
    apsl_model = app_models.DispersionAPSuggestionlist
    tree_model = app_models.DispersionTreeElement
    namespace = "dispersions"


class Dispersions_SuggestionlistTest(PermissionsMixIn, SettingsMixIn, base_tests.SuggestionlistTest):
    pass

class Dispersions_ScenarioTest(PermissionsMixIn, SettingsMixIn, base_tests.ScenarioTest):
    pass

class Dispersions_EventTest(PermissionsMixIn, SettingsMixIn, base_tests.EventTest):
    pass

class Dispersions_ProtectionLayerTest(PermissionsMixIn, SettingsMixIn, base_tests.ProtectionLayerTest):
    pass

class Dispersions_MeasureTest(PermissionsMixIn, SettingsMixIn, base_tests.MeasureTest):
    pass

class Dispersions_APSuggestionlistTest(PermissionsMixIn, SettingsMixIn, base_tests.APSuggestionlistTest):
    pass

class Dispersions_AttentionPointTest(PermissionsMixIn, SettingsMixIn, base_tests.AttentionPointTest):
    pass

class Dispersions_VersioningTest(PermissionsMixIn, SettingsMixIn, base_tests.VersioningTest):
    pass

class Dispersions_SearchTestCase(PermissionsMixIn, SettingsMixIn, base_tests.SearchTest):
    searchclasses = {
        'sl_search': app_models.Dispersions_SuggestionlistSearch,
        'sc_search': app_models.Dispersions_ScenarioSearch,
        'ev_search': app_models.Dispersions_EventSearch,
        'pl_search': app_models.Dispersions_ProtectionLayerSearch,
        'me_search': app_models.Dispersions_MeasureSearch,
        'apsl_search': app_models.Dispersions_APSuggestionlistSearch,
        'ap_search': app_models.Dispersions_AttentionPointSearch,
        }
