﻿"""
Tests for comments app
"""

import datetime

from django import test
from django.conf import settings
from django.contrib.auth.models import User, Permission, AnonymousUser
from django.contrib.contenttypes.models import ContentType

import time

import comments.comments_test.models as test_models
from comments import models as app_models
import comments.views as test_views
from utils import tests as utils_tests
from utils.versioning.models import Version, WorkVersion
from utils.versioning.versioning_test import models as vt_models

class CommentsTestCase(test.TestCase):
    fixtures = ['dummyusers',]

    def setUp(self):
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = to add, not to manage
        # 3 = only authorised to display
        # 4 = unauthorised to even display

        u = User.objects.get(pk=1)

        permissions = [
            ('display_versions','versioning','version'),
            ('display_comments','comments','comment'),
            ('write_comments','comments','comment'),
            ('manage_comments','comments','comment'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=2)
        permissions = [
            ('display_versions','versioning','version'),
            ('display_comments','comments','comment'),
            ('write_comments','comments','comment'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=3)
        permissions = [
            ('display_versions','versioning','version'),
            ('display_comments','comments','comment'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        s = test_models.CommentStreet(name='First')
        s.save(User.objects.get(pk=1))

class test_comments(CommentsTestCase):

    def test_views(self):
        csid = test_models.CommentStreet.objects.all()[0].id
        csvid = test_models.CommentStreet.objects.all()[0].v_id

        self.assertEquals(test_views.add_url_for_obj(test_models.CommentStreet.objects.get(pk=csid)), "/comments/add-for/comments_test/commentstreet/%s/" % csvid)
        self.assertEquals(test_views.display_url_for_obj(test_models.CommentStreet.objects.get(pk=csid)), "/comments/display-for/comments_test/commentstreet/%s/" % csvid)

    def test_comment_views(self):
        base_url = '/comments/'
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3,)

        csid = test_models.CommentStreet.objects.all()[0].id
        csvid = test_models.CommentStreet.objects.all()[0].v_id

        # setup a comment
        newitem = app_models.Comment(comment="Todo à", content_object=test_models.CommentStreet.objects.get(pk=csid))
        newitem.save(defaultuser)
        newitem = app_models.Comment(comment="C2", content_object=test_models.CommentStreet.objects.get(pk=csid))
        newitem.save(User.objects.get(pk=2))

        class comments(utils_tests.tablelist_TestDefinition):
            description = 'list of comments'
            url = base_url
            users = [1,2,3,]
            rowcount = 2
            # no general add for now
            contains = [{'url': base_url + 'add/', 'users': []},]

        class comments_filtered1(utils_tests.tablelist_filtered_TestDefinition):
            description = 'filtered list of comments 1'
            url = base_url
            users = [1,2,3,]
            filterdata = {"filter-user": "1",}
            # No redirect on unique here
##            redirect_url = base_url + '1/'
            rowcount = 1

        class comments_filtered2(utils_tests.tablelist_filtered_TestDefinition):
            description = 'filtered list of comments 2'
            url = base_url
            users = [1,2,3,]
            filterdata = {"filter-user": "3",}
            rowcount = 0

        class comments_report_select(utils_tests.TestDefinition):
            description = 'report of list of comments: select options'
            url = base_url + '?submit_pdf=submit_pdf&filter-user=1'
            users = [1,2,3]

        class comments_report(utils_tests.postonly_TestDefinition):
            description = 'pdf of list of comments'
            url = base_url
            users = [1,2,3]
            data = {
                "filter": "submit_pdf=submit_pdf&filter-user=1",
                "submit_pdf": "Generate pdf",
            }

        class comments_csv(utils_tests.postonly_TestDefinition):
            description = 'csv of list of comments'
            url = base_url
            users = [1,2,3]
            data = {
                "filter": "filter-user=1",
                "submit_csv": "Generate csv",
            }

        class comment1(utils_tests.TestDefinition):
            description = 'display comment 1'
            url = base_url + '1/'
            users = [1,2,3, ]
            contains = [
                {'url': base_url + '1/edit/', 'users': [1,]},
                {'url': base_url + '1/delete/', 'users': [1,]},
            ]

        class comment1_edit(utils_tests.edit_TestDefinition):
            description = 'edit comment 1'
            url = base_url + '1/edit/'
            users = [1,]
            data = {
                "comment": "Todo",
                "management-referer": base_url + '1/'
            }
            contains = [
                {'url': 'id_response', 'users': [1,]},
            ]
            redirect_url = base_url + '1/'

        class comment2(utils_tests.TestDefinition):
            description = 'display comment 2'
            url = base_url + '2/'
            users = [1,2,3, ]
            contains = [
                {'url': base_url + '2/edit/', 'users': [1,2,]},
                {'url': base_url + '2/delete/', 'users': [1,2,]},
            ]

        class comment2_edit(utils_tests.edit_TestDefinition):
            description = 'edit comment 2'
            url = base_url + '2/edit/'
            users = [1,2,]
            data = {
                "comment": "C2",
                "management-referer": base_url + '2/'
            }
            contains = [
                {'url': 'id_response', 'users': [1,]},
            ]
            redirect_url = base_url + '2/'

        class comment_add(utils_tests.add_TestDefinition):
            description = 'add a comment'
            url = base_url + 'add-for/comments_test/commentstreet/1/'
            users = [1,2]
            data = {
                    "management-referer": base_url + 'display-for/comments_test/commentstreet/%s/' % csvid,
                    "comment": "This sucks",
                }
            redirect_url = base_url + 'display-for/comments_test/commentstreet/%s/' % csvid
            def posttest(self, testcase, allowed):
                if allowed:
                    app_models.Comment.objects.order_by('-id').get_versionedobjects(version=WorkVersion)[0].delete(testcase.user, override=True)

        class comment1_delete(utils_tests.delete_TestDefinition):
            description = 'delete a comment by user 2'
            users = [1,2,]
            redirect_url = base_url + 'display-for/comments_test/commentstreet/%s/' % csvid
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = app_models.Comment(comment="Oh my", content_object=test_models.CommentStreet.objects.get(pk=csid))
                newitem.save(User.objects.get(pk=2))
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                # clean up if the item was not deleted
                if not allowed:
                    app_models.Comment.objects.get_versionedobject(id=self.pk, version=WorkVersion).delete(defaultuser, override=True)

        class comment2_delete(utils_tests.delete_TestDefinition):
            description = 'delete a comment by user 1'
            users = [1,]
            redirect_url = base_url + 'display-for/comments_test/commentstreet/%s/' % csvid
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = app_models.Comment(comment="Oh my", content_object=test_models.CommentStreet.objects.get(pk=csid))
                newitem.save(User.objects.get(pk=1))
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                # clean up if the item was not deleted
                if not allowed:
                    app_models.Comment.objects.get_versionedobject(id=self.pk, version=WorkVersion).delete(defaultuser, override=True)

        class comment1_report(utils_tests.TestDefinition):
            description = 'pdf report for comment'
            url = base_url + '1/pdf/'
            users = [1,2,3]

        class comment2_report(utils_tests.TestDefinition):
            description = 'pdf report for comment'
            url = base_url + '2/pdf/'
            users = [1,2,3]

        class comments_display_for(utils_tests.TestDefinition):
            description = 'display comments for street 1'
            url = base_url + 'display-for/comments_test/commentstreet/%s/' % csvid
            users = [1,2,3]
            contains = [
                {'url': base_url + 'add-for/comments_test/commentstreet/%s/' % csvid, 'users': [1,2]},
            ]


        tests = [
            comments(),
            comments_filtered1(),
            comments_filtered2(),
            comments_report_select(),
            comments_report(),
            comments_csv(),
            comment1(),
            comment1_edit(),
            comment2(),
            comment2_edit(),
            comment_add(),
            comment1_delete(),
            comment2_delete(),
            comment1_report(),
            comment2_report(),
            comments_display_for(),
        ]

        utils_tests.run_tests(self, users, tests)


    def test_templatetags(self):
        from django.template import Context, Template

        csid = test_models.CommentStreet.objects.all()[0].id

        d = {'STATIC_URL': 'x', 'object': test_models.CommentStreet.objects.get(pk=csid)}

        t = Template("""
            {% load comments_tags %}
            {% comments_link object %}
        """)
        response = t.render(Context(d))

class CommentsPublishTestCase(CommentsTestCase):
    def test_publish(self):
        """
        Test that when publishing a single object, the comments for that object are also published
        """
        user = User.objects.get(pk=1)
        csid = test_models.CommentStreet.objects.all()[0].id

        self.assertEquals(vt_models.Building.objects.count(), 0)
        self.assertEquals(vt_models.Building.objects.next_v_id(), 1)

        b1 = vt_models.Building(name='b1')
        b1.save(user)

        b2 = vt_models.Building(name='b2')
        b2.save(user)

        self.assertEquals(vt_models.Building.objects.count(), 2)
        self.assertEquals(vt_models.Building.objects.dirty().count(), 2)
        self.assertEquals(vt_models.Building.objects.filter(published_dirty=True, version__isnull=True).count(), 2)
        self.assertEquals(vt_models.Building.objects.filter(published=True).count(), 0)

        v1 = Version(name='v1', published=True)
        v1.save(user)

        self.assertEquals(vt_models.Building.objects.count(), 4)
        self.assertEquals(vt_models.Building.objects.dirty().count(), 0)
        self.assertEquals(vt_models.Building.objects.filter(published_dirty=True, version__isnull=True).count(), 0)
        self.assertEquals(vt_models.Building.objects.filter(published=True).count(), 2)

        # add comment to b1
        b1 = vt_models.Building.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        a = app_models.Comment(content_object=b1, comment='Nice')
        a.save(user)

        # add comment to b2
        b2 = vt_models.Building.objects.get_versionedobject(v_id = 2, version=WorkVersion)
        a = app_models.Comment(content_object=b2, comment='Nice')
        a.save(user)

        self.assertEquals(app_models.Comment.objects.count(), 2)
        self.assertEquals(app_models.Comment.objects.dirty().count(), 2)
        self.assertEquals(app_models.Comment.objects.filter(published_dirty=True, version__isnull=True).count(), 2)
        self.assertEquals(app_models.Comment.objects.filter(published=True).count(), 0)

        # create a version of b1
        time.sleep(0.01)
        v2 = Version(name='v2', published=True)
        v2.save(user, propagate=False)
        b2.publish(user, v2)

        self.assertEquals(app_models.Comment.objects.count(), 3)
        self.assertEquals(app_models.Comment.objects.dirty().count(), 1)
        self.assertEquals(app_models.Comment.objects.filter(published_dirty=True, version__isnull=True).count(), 1)
        self.assertEquals(app_models.Comment.objects.filter(published=True).count(), 1)

class CommentsSearchTestCase(CommentsTestCase):
    def test_searches(self):
        users = (0,1,2,3,4)

        defaultuser = User.objects.get(pk=1)
        csid = test_models.CommentStreet.objects.all()[0].id

        # setup a comment
        newitem = app_models.Comment(comment="Todo", content_object=test_models.CommentStreet.objects.get(pk=csid))
        newitem.save(defaultuser)

        class base_search(utils_tests.TestDefinition):
            description = ''
            searchclass = None
            searchitems = []
            users = [1,2,3,]
            def runtests(self, testcase, allowed, contains, notcontains):
                user = testcase.user or AnonymousUser()
                for item in self.searchitems:
                    res = allowed and item['result'] or 0
                    testcase.assertEquals(len(self.searchclass().search(item['query'], user, WorkVersion, None)), res)

        class search_comment(base_search):
            description = 'search comment'
            searchclass = app_models.Comments_CommentSearch
            searchitems = [
                { "query": 'odo', "result": 1},
                { "query": 'xxx', "result": 0},
            ]

        tests = [
            search_comment(self),
        ]

        utils_tests.run_tests(self, users, tests)
