# main items for hazards app
from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _
from hazards import models as app_models
from utils import permissions
from utils.versioning import versionedcrud
from utils.versioning import models as versioningmodels
from utils.versioning.session import get_current_version


def hazards_navigationlist(current=None, user=None):
    navigationlist = [
        {'name': 'substances',
            'url': reverse('hazards_substances'),
            'label': _('Substances')},
        {'name': 'reactions',
            'url': reverse('hazards_reactions'),
            'label': _('Reactions')},
    ]
    if app_models.show_matrix():
        navigationlist.append(
            {'name': 'matrix',
            'url': reverse('hazards_matrix'),
            'label': _('Matrix')},
        )
    nav = []
    for item in navigationlist:
        if current == item['name']:
            nav.append({'label': item['label']})
        else:
            nav.append({'label': item['label'], 'url': item['url']})
    return nav


class publish(versionedcrud.PublishObjectView):
    def lookup_object(self, *args, **kwargs):
        (self.version, self.specialversion) = get_current_version(self.request)
        return None

    def check_permission(self, object):
        c = permissions.check_permission(self.request.user, 'hazards.edit_hazards')
        if c:
            return c
        c = permissions.check_permission(self.request.user, 'versioning.edit_versions')
        if c:
            return c

    def get_name(self, object):
        return _('Hazards analysis')

    def save_form(self, form):
        name = form.cleaned_data['name']
        instance = versioningmodels.Version(name=name, published=True)
        instance.save(self.request.user, propagate=False)
        app_models.publish_hazards(self.request.user, instance)
        return self.object

    def add_to_extra_context(self, extra_context, object):
        extra_context['helptext'] = _('Publishing the hazards information will include the substance and reaction info.')
