from django import forms
from django.contrib.auth.models import User, Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q
from django.utils.translation import ugettext_lazy as _

from meetings import models as app_models
from utils import fields as utils_fields
from utils import lists as utils_lists
from utils import widgets as utils_widgets
from utils import sheets
from utils.versioning.versionedforms import VersionedForm, VersionedSheet

class MeetingForm(VersionedForm):
    participants = utils_fields.UserModelMultipleChoiceField(User.objects.none(),
        label=_('Participants'),
        required=False,
        widget = utils_widgets.FilteredSelectMultiple(verbose_name=_('participants'),
            is_stacked=False)
    )
    class Meta:
        model = app_models.Meeting
        attrs = {
            'subject': {'size':75},
            'description': {'rows':10,'cols':65},
        }
        widgets = {
            'description': utils_widgets.MarkItUpWidget()
        }

    def __init__(self, *args, **kwargs):
        super(MeetingForm, self).__init__(*args, **kwargs)
        _getparticipatepermission = Permission.objects.get_by_natural_key('participate_meetings', 'meetings', 'meeting')
        self.fields['participants'].queryset = User.objects.filter(
            Q(user_permissions=_getparticipatepermission)|Q(groups__permissions=_getparticipatepermission)
            ).distinct().order_by('last_name', 'first_name')

class MeetingSheet(VersionedSheet):
    participants = utils_fields.UserSheetManyToManyField(data='participants', name='participants', label=_('Participants'))
    description = utils_fields.MarkdownField(data = 'description', name='description', label=_('Description'))
    class Meta:
        model = app_models.Meeting

class MeetingFilterForm(utils_lists.FilterForm):
    """Custom filterform for the meetings list"""
    subject = forms.CharField(max_length=50, required=False, label=_('Subject'))
##    participant = forms.ModelChoiceField(queryset = User.objects.none(), required=False,
##        label=_('Participant'))

    class Meta:
        attrs = {
            'subject': {'size': 40},
            }

##    def __init__(self, *args, **kwargs):
##        super(MeetingFilterForm, self).__init__(*args, **kwargs)
##        _getparticipatepermission = Permission.objects.get_by_natural_key('participate_meetings', 'meetings', 'meeting')
##        self.fields['participant'].queryset = User.objects.filter(
##            Q(user_permissions=_getparticipatepermission)|Q(groups__permissions=_getparticipatepermission)|Q(is_superuser=True)
##            ).distinct()
