﻿"""
Tests for tasks app
"""

import datetime

from django import test
from django.conf import settings
from django.contrib.auth.models import User, Permission, AnonymousUser
from django.contrib.contenttypes.models import ContentType

import time

import tasks.tasks_test.models as test_models
from tasks import models as app_models
import tasks.views.task as test_views
from utils import tests as utils_tests
from utils.versioning.models import Version, WorkVersion
from utils.versioning.versioning_test import models as vt_models

class TasksTestCase(test.TestCase):
    fixtures = ['dummyusers',]

    def setUp(self):
        # give the users permissions

        # authorisations for users:
        # 1 = authorised
        # 2 = to get a task, not to edit
        # 3 = only authorised to display
        # 4 = unauthorised to even display

        u = User.objects.get(pk=1)

        permissions = [
            ('display_versions','versioning','version'),
            ('display_tasks','tasks','task'),
            ('edit_tasks','tasks','task'),
            ('get_tasks','tasks','task'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=2)
        permissions = [
            ('display_versions','versioning','version'),
            ('display_tasks','tasks','task'),
            ('get_tasks','tasks','task'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        u = User.objects.get(pk=3)
        permissions = [
            ('display_versions','versioning','version'),
            ('display_tasks','tasks','task'),
        ]
        for item in permissions:
            u.user_permissions.add(Permission.objects.get_by_natural_key(*item))

        s = test_models.TaskStreet(name='First')
        s.save(User.objects.get(pk=1))

class test_tasks(TasksTestCase):

    def test_views(self):
        csid = test_models.TaskStreet.objects.all()[0].id
        csvid = test_models.TaskStreet.objects.all()[0].v_id

        self.assertEquals(test_views.add_url_for_obj(test_models.TaskStreet.objects.get(pk=csid)), "/tasks/add-for/tasks_test/taskstreet/%s/" % csvid)
        self.assertEquals(test_views.display_url_for_obj(test_models.TaskStreet.objects.get(pk=csid)), "/tasks/display-for/tasks_test/taskstreet/%s/" % csvid)

    def test_task_views(self):
        base_url = '/tasks/'
        defaultuser = User.objects.get(pk=1)
        users = (0,1,2,3,)

        csid = test_models.TaskStreet.objects.all()[0].id
        csvid = test_models.TaskStreet.objects.all()[0].v_id

        # setup a task
        newitem = app_models.Task(task=u"Todo à", user=defaultuser, content_object=test_models.TaskStreet.objects.get(pk=csid))
        newitem.save(defaultuser)

        class tasks(utils_tests.tablelist_TestDefinition):
            description = 'list of tasks'
            url = base_url
            users = [1,2,3,]
            rowcount = 1
            # no general add for now
            contains = [{'url': base_url + 'add/', 'users': []},]

        class tasks_filtered1(utils_tests.tablelist_filtered_TestDefinition):
            description = 'filtered list of tasks 1'
            url = base_url
            users = [1,2,3,]
            filterdata = {"filter-user": "1",}
            # No redirect on unique here
##            redirect_url = base_url + '1/'
            rowcount = 1

        class tasks_filtered2(utils_tests.tablelist_filtered_TestDefinition):
            description = 'filtered list of tasks 2'
            url = base_url
            users = [1,2,3,]
            filterdata = {"filter-user": "2",}
            rowcount = 0

        class tasks_report_select(utils_tests.TestDefinition):
            description = 'report of list of tasks: select options'
            url = base_url + '?submit_pdf=submit_pdf&filter-user=2'
            users = [1,2,3]

        class tasks_report(utils_tests.postonly_TestDefinition):
            description = 'pdf of list of tasks'
            url = base_url
            users = [1,2,3]
            data = {
                "filter": "submit_pdf=submit_pdf&filter-user=2",
                "submit_pdf": "Generate pdf",
            }

        class tasks_csv(utils_tests.postonly_TestDefinition):
            description = 'csv of list of tasks'
            url = base_url
            users = [1,2,3]
            data = {
                "filter": "filter-user=2",
                "submit_csv": "Generate csv",
            }

        class task(utils_tests.TestDefinition):
            description = 'display task'
            url = base_url + '1/'
            users = [1,2,3, ]
            contains = [
                {'url': base_url + '1/edit/', 'users': [1,]},
                {'url': base_url + '1/delete/', 'users': [1,]},
            ]

        class task_edit(utils_tests.edit_TestDefinition):
            description = 'edit a task'
            url = base_url + '1/edit/'
            users = [1,]
            data = {
                "task": "Todo",
                "description": "",
                "result": "",
                "duedate": "",
                "completiondate": "",
                "user": 2,
                "management-referer": base_url + '1/'
            }
            redirect_url = base_url + '1/'
            def posttest(self, testcase, allowed):
                # clean up
                if allowed:
                    item = app_models.Task.objects.get_versionedobject(v_id=1, version=WorkVersion)
                    item.user = User.objects.get(pk=1)
                    item.save(defaultuser)

        class task_delete(utils_tests.delete_TestDefinition):
            description = 'delete a task'
            users = [1,]
            redirect_url = base_url + 'display-for/tasks_test/taskstreet/%s/' % csvid
            def pretest(self, testcase, allowed):
                # create a new item that we can delete
                newitem = app_models.Task(task="Oh my", content_object=test_models.TaskStreet.objects.get(pk=csid))
                newitem.save(defaultuser)
                self.pk = newitem.id
                self.url = base_url + '%s/delete/' % newitem.v_id
            def posttest(self, testcase, allowed):
                # clean up if the item was not deleted
                if not allowed:
                    app_models.Task.objects.get_versionedobject(id=self.pk, version=WorkVersion).delete(defaultuser, override=True)

        class task_report(utils_tests.TestDefinition):
            description = 'pdf report for task'
            url = base_url + '1/pdf/'
            users = [1,2,3]

        class tasks_display_for(utils_tests.TestDefinition):
            description = 'display tasks for street 1'
            url = base_url + 'display-for/tasks_test/taskstreet/%s/' % csvid
            users = [1,2,3]
            contains = [
                {'url': base_url + 'add-for/tasks_test/taskstreet/%s/' % csvid, 'users': [1,]},
            ]

        class tasks_add_for(utils_tests.add_TestDefinition):
            description = 'add a task'
            url = base_url + 'add-for/tasks_test/taskstreet/%s/' % csvid
            users = [1,]
            data = {
                    "management-referer": base_url + 'display-for/tasks_test/taskstreet/%s/' % csvid,
                    "task": "This sucks",
                }
            redirect_url = base_url + 'display-for/tasks_test/taskstreet/%s/' % csvid
            def posttest(self, testcase, allowed):
                if allowed:
                    app_models.Task.objects.order_by('-id').get_versionedobjects(version=WorkVersion)[0].delete(testcase.user, override=True)



        tests = [
            tasks(),
            tasks_filtered1(),
            tasks_filtered2(),
            tasks_report_select(),
            tasks_report(),
            tasks_csv(),
            task(),
            task_edit(),
            task_delete(),
            task_report(),
            tasks_display_for(),
            tasks_add_for(),
        ]

        utils_tests.run_tests(self, users, tests)


    def test_templatetags(self):
        from django.template import Context, Template

        csid = test_models.TaskStreet.objects.all()[0].id

        d = {'STATIC_URL': 'x', 'object': test_models.TaskStreet.objects.get(pk=csid)}

        t = Template("""
            {% load tasks_tags %}
            {% tasks_link object %}
        """)
        response = t.render(Context(d))

        d = {'STATIC_URL': 'x', 'user': User.objects.get(pk=1)}
        t = Template("""
            {% load tasks_tags %}
            {% get_tasks 5 as tasks %}
        """)
        response = t.render(Context(d))

class TasksPublishTestCase(TasksTestCase):
    def test_publish(self):
        """
        Test that when publishing a single object, the tasks for that object are also published
        """
        user = User.objects.get(pk=1)
        csid = test_models.TaskStreet.objects.all()[0].id

        self.assertEquals(vt_models.Building.objects.count(), 0)
        self.assertEquals(vt_models.Building.objects.next_v_id(), 1)

        b1 = vt_models.Building(name='b1')
        b1.save(user)

        b2 = vt_models.Building(name='b2')
        b2.save(user)

        self.assertEquals(vt_models.Building.objects.count(), 2)
        self.assertEquals(vt_models.Building.objects.dirty().count(), 2)
        self.assertEquals(vt_models.Building.objects.filter(published_dirty=True, version__isnull=True).count(), 2)
        self.assertEquals(vt_models.Building.objects.filter(published=True).count(), 0)

        v1 = Version(name='v1', published=True)
        v1.save(user)

        self.assertEquals(vt_models.Building.objects.count(), 4)
        self.assertEquals(vt_models.Building.objects.dirty().count(), 0)
        self.assertEquals(vt_models.Building.objects.filter(published_dirty=True, version__isnull=True).count(), 0)
        self.assertEquals(vt_models.Building.objects.filter(published=True).count(), 2)

        # add task to b1
        b1 = vt_models.Building.objects.get_versionedobject(v_id = 1, version=WorkVersion)
        a = app_models.Task(content_object=b1, task='Do this')
        a.save(user)

        # add task to b2
        b2 = vt_models.Building.objects.get_versionedobject(v_id = 2, version=WorkVersion)
        a = app_models.Task(content_object=b2, task='Do this')
        a.save(user)

        self.assertEquals(app_models.Task.objects.count(), 2)
        self.assertEquals(app_models.Task.objects.dirty().count(), 2)
        self.assertEquals(app_models.Task.objects.filter(published_dirty=True, version__isnull=True).count(), 2)
        self.assertEquals(app_models.Task.objects.filter(published=True).count(), 0)

        # create a version of b1
        time.sleep(0.01)
        v2 = Version(name='v2', published=True)
        v2.save(user, propagate=False)
        b2.publish(user, v2)

        self.assertEquals(app_models.Task.objects.count(), 3)
        self.assertEquals(app_models.Task.objects.dirty().count(), 1)
        self.assertEquals(app_models.Task.objects.filter(published_dirty=True, version__isnull=True).count(), 1)
        self.assertEquals(app_models.Task.objects.filter(published=True).count(), 1)

class TasksSearchTestCase(TasksTestCase):
    def test_searches(self):
        users = (0,1,2,3,4)

        defaultuser = User.objects.get(pk=1)
        csid = test_models.TaskStreet.objects.all()[0].id

        # setup a task
        newitem = app_models.Task(task="Todo", user=defaultuser, content_object=test_models.TaskStreet.objects.get(pk=csid))
        newitem.save(defaultuser)

        class base_search(utils_tests.TestDefinition):
            description = ''
            searchclass = None
            searchitems = []
            users = [1,2,3,]
            def runtests(self, testcase, allowed, contains, notcontains):
                user = testcase.user or AnonymousUser()
                for item in self.searchitems:
                    res = allowed and item['result'] or 0
                    testcase.assertEquals(len(self.searchclass().search(item['query'], user, WorkVersion, None)), res)

        class search_task(base_search):
            description = 'search task'
            searchclass = app_models.Tasks_TaskSearch
            searchitems = [
                { "query": 'odo', "result": 1},
                { "query": 'xxx', "result": 0},
            ]

        tests = [
            search_task(self),
        ]

        utils_tests.run_tests(self, users, tests)
